﻿using mvvm.Code;
using mvvm.Views;
using PropertyChanged;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Data;

/* Created by Jan Kobersky - 3/9/2016 6:38:15 PM */
namespace mvvm.ViewModels
{
    [ImplementPropertyChanged]
    public class TokenViewModel
    {
        public string Token { get; private set; } // token, ktery pridje z API a dale je vyuzit pri navigaci

        public SimpleActionCommand RequestToken { get; } // command pro ovladani tlacitka na ziskani tokenu
        public SimpleActionCommand Continue { get; } // command pro ovladani tlacitka pro prechod na dalsi obrazovku

        private Frame _frame;
        private ApiService _apiService = new ApiService();

        public TokenViewModel(Frame frame)
        {
            _frame = frame;
            RequestToken = new SimpleActionCommand(RetrieveToken);
            RequestToken.CanAction = true;

            Continue = new SimpleActionCommand(GoToText);
        }

        // diky naming convention Fody.PropertyChanged je tato metoda automaticky zavolana pri zmene property Token
        public void OnTokenChanged()
        {
            RequestToken.CanAction = string.IsNullOrEmpty(Token);

            Continue.CanAction = Token != null;
        }

        private void GoToText()
        {
            _frame.Navigate(typeof(TextView), Token);
        }
        
        private async void RetrieveToken()
        {
            Token = await _apiService.GetToken();
        }
    }

    // ukazkova implementace commandu pro tlacitka.
    // tento command dostane v konstruktoru delegate, ktereho bude volat pri vyvolani akce
    public class SimpleActionCommand : ICommand
    {
        // ICommand implementace
        public event EventHandler CanExecuteChanged;

        private bool _canAction;
        public bool CanAction
        {
            get
            {
                return _canAction;
            }

            set
            {
                _canAction = value;
                CanExecuteChanged?.Invoke(this, new EventArgs()); // zavolani eventt, pokud jej nekdo posloucha (neni null) s prazdnym argumentem
            }
        }
        public Action Action { get; }

        public SimpleActionCommand(Action action)
        {
            Action = action;
        }

        // ICommand implementace
        public bool CanExecute(object parameter)
        {
            return CanAction;
        }

        // ICommand implementace
        public void Execute(object parameter)
        {
            Action?.Invoke();
        }
    }

    // converter na prevod stringu na visibility
    public class StringEmptyOrNullConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, string language)
        {
            // pokud neni predany objekt prazdny string nebo null, element bude zorbazen, v opacnem pripade schovan
            return !string.IsNullOrEmpty(value as string) ? Visibility.Visible : Visibility.Collapsed;
        }

        // convertback v tomto pripade neni vyzadovan
        public object ConvertBack(object value, Type targetType, object parameter, string language)
        {
            throw new NotImplementedException();
        }
    }
}
